@@ -21,8 +21,11 @@
 
 #include <kdebug.h>
 
-IPod::IPod()
-    : itunesdb(true)
+using namespace itunesdb;
+
+IPod::IPod(const QString& mountPoint, const QString& device)
+    :   ipodBase(mountPoint), devicename(device), itunesdb(true),
+        deviceDetails(NULL), sysInfo(NULL), replayingLog( false )
 {
 }
 
@@ -30,16 +33,46 @@ IPod::IPod()
 IPod::~IPod()
 {
     itunesdb.clear();
+    delete deviceDetails;
+    delete sysInfo;
+    kdDebug() << "IPod at " << ipodBase << " cleaned up" << endl;
 }
 
-bool IPod::open(const IPodMountPoint& mountpoint) {
-    ipodBase = mountpoint.getMountPoint();
+bool IPod::open() {
+    kdDebug() << "IPod::open(" << ipodBase << ", " << devicename << ");" << endl;
+
+    sysInfo = new IPodSysInfo(ipodBase);
+    if ( !sysInfo->load() ) {
+        delete sysInfo;
+        sysInfo = NULL;
+        return false;
+    }
+
+    if (!itunesdb.open(ipodBase)) {
+        return false;
+    }
+
+    deviceDetails = new IPodDeviceDetails(ipodBase + IPodSysInfo::iPodControlDir + "/iTunes/DeviceInfo");
+    if ( ! deviceDetails->load() ) {
+        delete deviceDetails;
+        deviceDetails = NULL;
+    }
+
     logfileentrypos = 0;
     locked = false;
-    pendingchanges = false;
-    return itunesdb.open(ipodBase);
+    dirty = false;
+
+    replayLog();
+
+    return true;
 }
 
+
+void IPod::setDirty() {
+    dirty = true;
+}
+
+
 bool IPod::isOpen() {
     return itunesdb.isOpen();
 }
@@ -51,6 +84,10 @@ bool IPod::isStillConnected() {
 void IPod::close() {
     flushLog();
     itunesdb.clear();
+    delete sysInfo;
+    delete deviceDetails;
+    sysInfo = NULL;
+    deviceDetails = NULL;
 }
 
 
@@ -60,26 +97,43 @@ bool IPod::ensureConsistency() {
         flushLog();
         return false;
     }
-    
+
     replayLog();
-        
+
     return true;
 }
 
 
+QString IPod::getRealPath( QString pathinfo ) const {
+    QFile realPath( ipodBase + pathinfo.replace( ":", "/") );
+    if ( !realPath.exists() ) {
+        // umm, try to fix the path the simple way, and let the slave gen an error msg if this fails
+        QString pathToFix( realPath.name() );
+        QDir musicDir( getSysInfo()->getMusicDir() );
+        QString replacement( musicDir.path() + "/" + musicDir[0][0] );
+        realPath.setName( pathToFix.replace(0, replacement.length(), replacement) );
+    }
+    // TODO if that still doesn't work try to do it by path components
+
+    return realPath.name();
+}
+
+
 TrackMetadata IPod::createNewTrackMetadata() {
     Q_UINT32 trackid = itunesdb.getMaxTrackID();
-    while (itunesdb.getTrackByID( ++trackid) != NULL);    // look for the next free number
-    
+    while (itunesdb.getTrackByID( ++trackid) != NULL) { // look for the next free number
+        if (trackid >= 0xEFFFFFFF) trackid = 2000;    // start anew when max tracknumber is reached
+    }
+
     TrackMetadata track( trackid);
     // calculate the directory
-    Q_UINT32 dir = trackid % 20;
-    
+    QString dir = getSysInfo()->getMusicDir()[ trackid % sysInfo->getNumTrackFileDirs() ];
+
     // form the trackpath
     QString trackpath;
-    trackpath.sprintf( ":iPod_Control:Music:F%02d:%s", dir, (QString("kpod") + QString::number(trackid)).latin1());
+    trackpath.sprintf( ":iPod_Control:Music:%s:%s", dir.latin1(), (QString("kpod") + QString::number(trackid)).latin1());
     track.setPath(trackpath);
-    
+
     return track;
 }
 
@@ -87,13 +141,13 @@ IPod::IPodError IPod::deleteArtist(const QString& artistname, bool log) {
     if (!itunesdb.removeArtist(artistname)) {
         return Err_NotEmpty;    // dirty since false is also emmitted if the artist doesn't exist
     }
-    
+
     if (log) {
         QStringList actions;
         actions << artistname;
         appendLogEntry(ACT_DELETE_ARTIST, actions);
     }
-    
+
     return Err_None;
 }
 
@@ -103,47 +157,83 @@ IPod::IPodError IPod::deleteArtist(const QString& artistname, bool log) {
 IPod::IPodError IPod::renameAlbum(const QString& artistname, const QString& title, const QString& newartistname, const QString& newtitle, bool log)
 {
     kdDebug() << "IPod::renameAlbum() " << title << endl;
-    
+
     if (!itunesdb.isOpen()) {
         return Err_NotOpen;
     }
     if (itunesdb.getAlbum(newartistname, newtitle)) {    // does it already exist?
         return Err_AlreadyExists;
     }
-    
+
     TrackList * album = itunesdb.getAlbum(artistname, title);
     if (album == NULL) {
         return Err_DoesNotExist;
     }
-    
+
     if(!itunesdb.renameAlbum(*album, newartistname, newtitle)) {
         kdDebug() << "IPod::renameAlbum() issued an internal error" << endl;
         return Err_Internal;
     }
-        
+
     if (log) {
         QStringList actions;
         actions << artistname << title << newartistname << newtitle;
         appendLogEntry(ACT_RENAME_ALBUM, actions);
     }
-    
-    pendingchanges = true;
-    
+
+    setDirty();
+
     kdDebug() << "IPod::renameAlbum() finished" << endl;
     return Err_None;
 }
 
 
+IPod::IPodError IPod::renameArtist(const QString& artistname, const QString& newartistname, bool log) {
+    kdDebug() << "IPod::renameArtist() " << artistname << endl;
+    if (!itunesdb.isOpen()) {
+        return Err_NotOpen;
+    }
+
+    if ( artistname == newartistname ) {
+        return Err_None;
+    }
+
+    Artist * oldartist = itunesdb.getArtistByName(artistname);
+    if ( oldartist == NULL ) {
+        return Err_DoesNotExist;
+    }
+
+    for( ArtistIterator albumiterator( *oldartist ); albumiterator.current(); ) {
+        kdDebug() << "renaming album " << (*albumiterator.current()).getTitle() << endl;
+        if ( ! itunesdb.renameAlbum( *albumiterator.current(), newartistname, QString::null ) ) {
+            kdDebug() << "IPod::renameAlbum() issued an internal error" << endl;
+            return Err_Internal;
+        }
+    }
+
+    itunesdb.removeArtist( artistname );
+    if ( log ) {
+        QStringList actions;
+        actions << artistname << newartistname;
+        appendLogEntry( ACT_RENAME_ARTIST, actions );
+    }
+
+    setDirty();
+
+    kdDebug() << "IPod::renameArtist() finished" << endl;
+    return Err_None;
+}
+
 IPod::IPodError IPod::deleteAlbum(const QString& artistname, const QString& title, bool log) {
     TrackList * album = getAlbum(artistname, title);
     if (album == NULL)
         return Err_DoesNotExist;
-    
+
     TrackList::Iterator trackiter = album->getTrackIDs();
     while (trackiter.hasNext()) {
         TrackMetadata * track = getTrackByID(trackiter.next());
         album->removeTrackAt(trackiter);
-        
+
         if (track == NULL) {
             continue;
         }
@@ -152,17 +242,18 @@ IPod::IPodError IPod::deleteAlbum(const QString& artistname, const QString& titl
             QFile::remove(filename);
         itunesdb.removeTrack(track->getID());
     }
-    
+
     Artist * artist = getArtistByName(artistname);
-    if (artist != NULL)
+    if ( artist != NULL ) {
         artist->remove(album->getTitle());
-    
+    }
     if (log) {
         QStringList actions;
         actions << artistname << title;
         appendLogEntry(ACT_REM_ALBUM, actions);
+        sysInfo->refreshDiskUsageStats();
     }
-        
+
     return Err_None;
 }
 
@@ -170,6 +261,16 @@ TrackList * IPod::getAlbum(const QString &artistname, const QString &albumname)
     return itunesdb.getAlbum(artistname, albumname);
 }
 
+/**
+ * returns the Track found by the given information or NULL if no such Track could be found
+ * @param artistname the name of the artist
+ * @param albumname the name of the album
+ * @param title the title of the track
+ */
+TrackMetadata * IPod::findTrack(const QString& artistname, const QString& albumname, const QString& title) const {
+    return itunesdb.findTrack( artistname, albumname, title );
+}
+
 
 /*!
     \fn IPod::renamePlaylist(const QString& name, const QString& newname)
@@ -191,14 +292,15 @@ IPod::IPodError IPod::renamePlaylist(const QString& title, const QString& newtit
     playlist->setTitle(newtitle);
     itunesdb.handlePlaylist(*playlist);
     delete playlist;
-    
+
     if (log) {
         QStringList actions;
         actions << title << newtitle;
         appendLogEntry( ACT_RENAME_PLAYLIST, actions);
     }
-    pendingchanges = true;
-    
+
+    setDirty();
+
     return Err_None;
 }
 
@@ -208,11 +310,13 @@ IPod::IPodError IPod::renamePlaylist(const QString& title, const QString& newtit
  */
 IPod::IPodError IPod::deletePlaylist(const QString& title, bool log)
 {
-    if(!itunesdb.removePlaylist(title, true))
+    if(!itunesdb.removePlaylist(title, true)) {
         return Err_DoesNotExist;
-    
+    }
     if (log) appendLogEntry(ACT_REM_PLAYLIST, QStringList(title));
-    pendingchanges = true;
+
+    setDirty();
+
     return Err_None;
 }
 
@@ -239,19 +343,18 @@ QStringList * IPod::getArtists( QStringList &buffer) const
     return itunesdb.getArtists(buffer);
 }
 
-QString IPod::getName() {
-    Playlist * mainlist = itunesdb.getMainplaylist();
-    if(mainlist != NULL)
-        return mainlist->getTitle();
-    else
-        return QString();
+QString IPod::getName() const {
+    if ( deviceDetails != NULL ) {
+        return deviceDetails->getName();
+    }
+    return itunesdb.getMainListTitle();
 }
 
 void IPod::setName(const QString& name) {
     Playlist * mainlist = itunesdb.getMainplaylist();
     if(mainlist != NULL) {
         mainlist->setTitle(name);
-        pendingchanges = true;
+        setDirty();
     }
 }
 
@@ -264,27 +367,75 @@ IPod::IPodError IPod::createPlaylist(const QString& playlisttitle, bool log) {
     if(itunesdb.getPlaylistByTitle(playlisttitle) != NULL) {
         return Err_AlreadyExists;
     }
-    
+
     Playlist playlist;
     playlist.setTitle(playlisttitle);
     itunesdb.handlePlaylist(playlist);
-    
-    if (log) appendLogEntry( ACT_ADD_PLAYLIST, QStringList(playlist.getTitle()));
-    pendingchanges = true;
-    
+
+    if ( log ) {
+        appendLogEntry( ACT_ADD_PLAYLIST, QStringList(playlist.getTitle()));
+    }
+    setDirty();
+
     return Err_None;
 }
 
+IPod::IPodError IPod::createArtist( const QString& artistname, bool log ) {
+    if ( itunesdb.getArtistByName( artistname ) ) {
+        return Err_AlreadyExists;
+    }
+
+    Artist * artist = itunesdb.getArtistByName( artistname, true );
+    if ( !artist ) {
+        return Err_Internal;
+    }
+
+    if ( log ) {
+        appendLogEntry( ACT_CREATE_ARTIST, QStringList( artistname ));
+    }
+
+    setDirty();
+
+    return Err_None;
+}
+
+IPod::IPodError IPod::createAlbum(const QString& artistName, const QString& albumName, bool log ) {
+    Artist * artist = getArtistByName( artistName );
+    if ( !artist ) {
+        return Err_DoesNotExist;
+    }
+
+    if ( artist->find ( albumName ) ) {
+        return Err_AlreadyExists;
+    }
+    TrackList * album = new TrackList();
+    album->setTitle( albumName );
+    artist->insert( albumName, album );
+
+    if ( log ) {
+        QStringList actions;
+        actions << artistName << albumName;
+        appendLogEntry( ACT_CREATE_ALBUM, actions);
+    }
+
+    setDirty();
+
+    return Err_None;
+}
 
 void IPod::writeItunesDB() {
+    lock(true);
     itunesdb.writeDatabase();
     flushLog();
+    unlock();
 }
 
 
 void IPod::writeItunesDB(const QString& filename) {
+    lock(true);
     itunesdb.writeDatabase(filename);
     flushLog();
+    unlock();
 }
 
 TrackMetadata * IPod::getTrackByID(const Q_UINT32 id) const {
@@ -302,16 +453,16 @@ IPod::IPodError IPod::addTrackToPlaylist(const TrackMetadata& track, const QStri
     if(playlist == NULL) {
         return Err_DoesNotExist;
     }
-    
+
     playlist->addPlaylistItem(track);
-    pendingchanges = true;
-    
+    setDirty();
+
     if (log) {
         QStringList actions;
-        actions << playlist->getTitle() << QString::number(track.getID());
+        actions << playlist->getTitle() << QString::number( track.getID(), 36 );
         appendLogEntry(ACT_ADD_TO_PLAYLIST, actions);
     }
-    
+
     return Err_None;
 }
 
@@ -320,44 +471,45 @@ IPod::IPodError IPod::removeFromPlaylist(Q_UINT32 position, const QString& playl
     if(playlist == NULL) {
         return Err_DoesNotExist;
     }
-    
+
     playlist->setTrackIDAt(position, LISTITEM_DELETED);
-    pendingchanges = true;
-    
+    setDirty();
+
     if (log) {
         QStringList actions;
-        actions << playlist->getTitle() << QString::number(position);
+        actions << playlist->getTitle() << QString::number( position, 36 );
         appendLogEntry(ACT_REM_FROM_PLAYLIST, actions);
     }
-    
+
     return Err_None;
 }
 
 
-void IPod::addTrack(const TrackMetadata& track, bool log) {
+void IPod::addTrack(TrackMetadata& track, bool log) {
     itunesdb.addTrack(track);
-    
+
     if (log) {
         QStringList actions;
         actions = track.toLogEntry(actions);
         appendLogEntry( ACT_ADD_TRACK, actions);
+        sysInfo->refreshDiskUsageStats();
     }
-    
-    pendingchanges = true;
+
+    setDirty();
 }
 
 IPod::IPodError IPod::moveTrack(TrackMetadata& track, const QString& newartist, const QString& newalbum, bool log) {
     if (!itunesdb.moveTrack(track, newartist, newalbum)) {
         return Err_DoesNotExist;
     }
-    
+
     if (log) {
         QStringList actions;
-        actions << QString::number(track.getID()) << newartist << newalbum;
+        actions << QString::number( track.getID(), 36 ) << newartist << newalbum;
         appendLogEntry(ACT_MOV_TRACK, actions);
     }
-    pendingchanges = true;
-    
+    setDirty();
+
     return Err_None;
 }
 
@@ -368,20 +520,21 @@ IPod::IPodError IPod::deleteTrack(Q_UINT32 trackid, bool log) {
 
     if (log) {
         QStringList actions;
-        actions << QString::number(trackid);
+        actions << QString::number( trackid, 36 );
         appendLogEntry(ACT_REM_TRACK, actions);
+        sysInfo->refreshDiskUsageStats();
     }
-    
-    pendingchanges = true;
+
+    setDirty();
     return Err_None;
 }
 
 
 void IPod::lock(bool write_lock) {
-    if(write_lock) {
+    // if(write_lock) {
         itunesdb.lock(write_lock);
         locked = true;
-    }
+    // }
 }
 
 bool IPod::isLocked() {
@@ -390,10 +543,10 @@ bool IPod::isLocked() {
 
 
 void IPod::unlock() {
-    if(locked) {
+    // if(locked) {
         itunesdb.unlock();
         locked = false;
-    }
+    // }
 }
 
 
@@ -407,34 +560,35 @@ bool IPod::appendLogEntry(IPod::LogActionType type, const QStringList& values)
     if (!logfile.open(IO_ReadWrite | IO_Append)) {
         return false;
     }
-    
+
     if (!isLocked()) {
         lock(true);
         _unlock_ = true;
     }
-    
+
     QByteArray logentry;
     QDataStream stream(logentry, IO_WriteOnly);
     stream.setByteOrder(QDataStream::LittleEndian);
     for (QStringList::const_iterator value_it = values.constBegin(); value_it != values.constEnd(); ++value_it) {
         stream << *value_it;
     }
-    
+
     QDataStream logstream(&logfile);
     logstream.setByteOrder(QDataStream::LittleEndian);
     logstream << type;
     logstream << logentry;
-    
+
     logstream.unsetDevice();
-    
+
     logfileentrypos++;
-    
+
     logfile.flush();
     logfile.close();
 
-    if(_unlock_)
+    if(_unlock_) {
         unlock();
-        
+    }
+
     return true;
 }
 
@@ -450,33 +604,34 @@ void IPod::replayLog()
         lock(false);
         _unlock_ = true;
     }
-    
+
     // kdDebug() << "IPod::replayLog() locked!" << endl;
-    
+
     QFile logfile(getLogfileName());
     if (!logfile.open(IO_ReadOnly)) {
         if(_unlock_)
             unlock();
         return;
     }
-    
+
+    replayingLog = true;
     // kdDebug() << "IPod::replayLog() logfile opened!" << endl;
-    
+
     QDataStream logstream(&logfile);
     logstream.setByteOrder(QDataStream::LittleEndian);
-    
+
     // ignore the changes we already know about
     for (uint i= 0; i< logfileentrypos; i++) {
         Q_UINT32 type;
         QByteArray buffer;
         if( logstream.atEnd()) {    // ick
             logfileentrypos= i;
-            break;                                            
+            break;
         }
         logstream >> type;
         logstream >> buffer;
     }
-        
+
     while (!logstream.atEnd()) {    // read new log entries
         QByteArray entrydata;
         Q_UINT32 type;
@@ -484,16 +639,16 @@ void IPod::replayLog()
 
         logstream >> type;
         logstream >> entrydata;
-        
+
         if (type >= NUM_ACTIONS) {
             continue;
         }
-        
+
         logfileentrypos++;
-        
+
         if( entrydata.isEmpty())
             continue;
-        
+
         // parse entry elements
         QDataStream entrystream(entrydata, IO_ReadOnly);
         entrystream.setByteOrder( QDataStream::LittleEndian);
@@ -502,7 +657,7 @@ void IPod::replayLog()
             entrystream >> value;
             values.push_back(value);
         }
-        
+
         switch( type) {    // handle logfile entry
         case ACT_ADD_PLAYLIST:
             // add playlist
@@ -531,9 +686,9 @@ void IPod::replayLog()
             }
             break;
         case ACT_ADD_TO_PLAYLIST: {
-            if (values.size()> 1) {
+            if ( values.size() > 1 ) {
                 bool conversion_ok= true;
-                Q_UINT32 trackid= values[ 1].toUInt(&conversion_ok);
+                Q_UINT32 trackid= values[ 1].toUInt( &conversion_ok, 36 );
                 if(conversion_ok) {
                     TrackMetadata * track = getTrackByID( trackid);
                     if(track != NULL)
@@ -545,7 +700,7 @@ void IPod::replayLog()
         case ACT_REM_FROM_PLAYLIST:
             if (values.size()> 1) {
                 bool conversion_ok = true;
-                int tracknum = values[ 1].toUInt(&conversion_ok);
+                int tracknum = values[ 1].toUInt( &conversion_ok, 36 );
                 if(conversion_ok)
                     removeFromPlaylist(tracknum, values[0], false);
             }
@@ -553,15 +708,15 @@ void IPod::replayLog()
         case ACT_ADD_TRACK:
             if (values.size() > 0) {
                 TrackMetadata track;
-                if(track.readFromLogEntry( values)) {
-                    addTrack(track, false);
+                if ( track.readFromLogEntry( values ) ) {
+                    addTrack( track, false );
                 }
             }    // ignore otherwise
             break;
         case ACT_MOV_TRACK:
             if (values.size() > 2) {
                 bool conversion_ok = true;
-                int trackid = values[0].toUInt(&conversion_ok);
+                int trackid = values[0].toUInt( &conversion_ok, 36 );
                 if (!conversion_ok)
                     break;
                 TrackMetadata * track = getTrackByID(trackid);
@@ -573,7 +728,7 @@ void IPod::replayLog()
         case ACT_REM_TRACK: {
             if (values.size() > 0) {
                 bool conversion_ok = true;
-                int trackid = values[ 0].toUInt(&conversion_ok);
+                int trackid = values[ 0].toUInt( &conversion_ok, 36 );
                 if(conversion_ok)
                     deleteTrack(trackid, false);
             }    // ignore otherwise
@@ -585,13 +740,34 @@ void IPod::replayLog()
             }    // ignore otherwise
             }
             break;
+        case ACT_RENAME_ARTIST: {
+            if ( values.size() > 1 ) {
+                renameArtist( values[ 0 ], values[ 1 ], false );
+            }
+            }
+            break;
+        case ACT_CREATE_ARTIST: {
+            if ( values.size() > 0 ) {
+                createArtist( values[ 0 ], false );
+            }
+            }
+            break;
+        case ACT_CREATE_ALBUM: {
+            if ( values.size() > 1 ) {
+                createAlbum( values[ 0 ], values[ 1 ], false );
+            }
+            }
+            break;
         default:
             break;
         }
     }
-    
-    if(_unlock_)
+
+    replayingLog = false;
+
+    if(_unlock_) {
         unlock();
+    }
 }
 
 
